%% You can run this file in cell mode: each cell (starting with %%) can be run by pressing CTRL+ENTER inside it

%% Approximate Q-learning

% approximator config 
acfg = struct;
acfg.enhanced = 1;      % just an internal flag, needs to be on to properly process options below
acfg.type = 'rbfdisc';  % approximator type: RBFs over state space, discretization over action space
acfg.N = [15, 11];      % grid size for each of the two state variables
acfg.M = 3;             % discretization grid size

% configuration parameters of the algorithm
cfg = struct;
cfg.problem = 'ipsetup_problem';        % the problem to solve
cfg.model_params = {[], [], 'Ts=.05'};  % model settings: notably, sampling time of .05 seconds
cfg.gamma = 0.98;               % discount factor
cfg.reset = [-pi, 0]';          % always reset to this state in the beginning of trials
cfg.approx = acfg;              % take the approximator config from above
cfg.eps = -1;                   % disregard convergence test (by using negative threshold)
cfg.maxtime  = 350;             % total simulated time
cfg.trialtime = 10;             % time per trial => we have maxtime/trialtime=35 trials
cfg.alpha = 0.5;                % initial alpha
cfg.alphadecay = 1;             % decay rate per trial (with 1, alpha stays constant)
cfg.explor = 1;                 % initial exploration rate
cfg.explordecay = 0.85;          % decay rate per trial
cfg.lambda = 0.7;                % eligibility trace decay rate
cfg.resetetronexplore = 0;      % do not reset trace on exploratory actions (maximize information transfer)
% output config
cfg.datadir = pwd;              % save to the current dir
cfg.datafile = 'ip_approxqlearn';% under this file name
% use visualization -- set visualize to 0 to disable
cfg.visualize = 3;              % visualize each step
cfg.viscfg.pause = 0.001;       % very short pause per step
cfg.viscfg.figsize = [450 600]; % figure size for visualization

% run the algorithm
cfg.run = 1;
approxqlearn(cfg);              

% look at the controlled trajectories at some trials
% here, traj means "plot trajectories", and "trialskip" means one of each 5 trials is shown
approxqlearn('traj datafile=ip_approxqlearn trialskip=4 verb=0');


%% Online LSPI

% approximator config -- same as before
acfg = struct;
acfg.enhanced = 1;      % just an internal flag, needs to be on to properly process options below
acfg.type = 'rbfdisc';  % RBFs over state space, discretization over action space
acfg.N = [15, 11];      % grid size for each of the two state variables
acfg.M = 3;             % discretization grid size

% configuration parameters -- only those different from above are commented
cfg = struct;
cfg.problem = 'ipsetup_problem';
cfg.model_params = {[], [], 'Ts=.05'}; 
cfg.gamma = 0.98;
cfg.reset = 'fixed';            % always reset in the beginning of trials to...        
cfg.x0 = [-pi, 0]';             % this state value
cfg.approx = acfg;              
cfg.maxtime  = 50;             
cfg.trialtime = 10;             
cfg.explor = 1;                 
cfg.explordecay = 0.85^(1/10);  % decay rate PER SECOND
    % since it's per second, to get the same effect as the 0.85 per trial above we use a larger
    % decay rate: specifically 0.85^(1/trial length in seconds), i.e., 10
cfg.dupd = .05*100;             % improve policy each 100 samples (with update interval Ts * 100)
% output config
cfg.datadir = pwd;              
cfg.datafile = 'ip_lspionline';
% use visualization -- set visualize to 0 to disable
cfg.visualize = 10;             % this means: only visualize system steps
cfg.viscfg.pause = 0.001;       
cfg.viscfg.figsize = [450 600];

% run the algorithm
cfg.run = 1;
lspionline(cfg);              

% look at the controlled trajectories at some trials
% here, the interval between trajectories is given in seconds, so to get 4 trials, it's 40 seconds
lspionline('traj datafile=ip_lspionline dtraj=40');


